require "nerve"

module Doorkeepr
  module Nerve
    class << self
      def publish
        puts "Publishing Server with #{Doorkeepr.config.inspect}"
        ap config
        @doorkeepr = ::Nerve::Nerve.new config
        marshal
        at_exit do
          @threads.each{|t|t.kill}
        end
      end
      def doorkeepr
        @doorkeepr
      end
      def nerve
      end
      def marshal
        puts 'Doorkeepr: publish!'
        @threads ||= []
        @threads << Thread.new { doorkeepr.run }
      end
      def service uri="/doorkeepr",path:doorkeepr_path
        {
          host: host,
          port: port,
          reporter_type: "zookeeper",
          zk_hosts: zk_hosts,
          zk_path:  path,
          check_interval: 5,
          checks:   [{timeout:0.2,rise:2,fall:3,type:"http",uri:uri}]
        }
      end
      def additional_paths
        paths   = Doorkeepr.paths
        paths ||= []
        paths.collect do |path|
          [
            Digest::MD5.hexdigest(path),
            service(uri,path:path)
          ]
        end.to_h
      end
      def config
        {
          instance_id: instance_id,
          services: {
            application => service,
            "uplink" => service(uri,path:uplink_path)
          }.merge(additional_paths)
        }.with_indifferent_access
      end
      def uplink_path
        @uplink_path   = Doorkeepr.config.uplink
        @uplink_path ||= "/nerve/#{Doorkeepr.config.customer}/#{Doorkeepr.config.application}/#{Doorkeepr.config.service}/services"
      end
      def doorkeepr_path
        @doorkeepr_path   = Doorkeepr.config.path
        @doorkeepr_path ||= "/doorkeepr/publish"
      end
      def host
        @host   = Doorkeepr.config.host
        @host ||= "localhost"
      end
      def port
        @port   = Doorkeepr.config.port
        @port ||= 3000
      end
      def uri
        @uri   = "/#{::OkComputer.mount_at}" if defined? ::OkComputer
        @uri ||= "/"
      end
      def instance_id
        @instance_id ||= SecureRandom.hex(8)
      end
      def zk_hosts
        return [ Boutons.zookeeper.s("%i:%p") ] if defined? Boutons and Boutons.respond_to? :zookeeper
        return Resolv::Consul.services(:zookeeper).map{|srv| "#{srv.address}:#{srv.port}" } if defined? Resolv::Consul
        [ "localhost:2181" ]
      end
      def application
        @application   = Doorkeepr.config.url
        @application ||= "rails.dev"
      end
    end
  end
end